package de.infa.plugins.bgloc;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.util.Log;
import android.widget.Toast;

import com.getcapacitor.JSArray;
import com.getcapacitor.JSObject;
import com.getcapacitor.PermissionState;
import com.getcapacitor.Plugin;
import com.getcapacitor.PluginCall;
import com.getcapacitor.PluginMethod;
import com.getcapacitor.annotation.CapacitorPlugin;
import com.getcapacitor.annotation.Permission;
import com.getcapacitor.annotation.PermissionCallback;
import com.google.android.gms.common.util.ArrayUtils;
import com.google.android.gms.location.LocationRequest;

import org.jetbrains.annotations.NotNull;

import java.util.Objects;

import de.infa.plugins.bgloc.service.HttpService;
import de.infa.plugins.bgloc.service.LocationService;
import de.infa.plugins.bgloc.service.PluginReceiver;
import de.infa.plugins.bgloc.service.configuration.PluginConfigurationReader;
import de.infa.plugins.bgloc.service.configuration.ServiceConfiguration;
import de.infa.plugins.bgloc.service.configuration.SessionConfiguration;
import de.infa.plugins.bgloc.service.exceptions.InvalidOperationException;
import de.infa.plugins.bgloc.service.exceptions.ServiceConfigurationException;
import de.infa.plugins.bgloc.service.storage.SQLiteCleanupIdsOptions;
import de.infa.plugins.bgloc.service.storage.SQLiteCleanupSessionOptions;
import de.infa.plugins.bgloc.service.storage.SQLiteFlagOptions;
import de.infa.plugins.bgloc.service.storage.SQLiteGetLocationsOptions;
import de.infa.plugins.bgloc.service.storage.SQLiteGetLocationsRecipient;
import de.infa.plugins.bgloc.service.storage.SQLiteJSObjectReader;
import de.infa.plugins.bgloc.service.storage.SQLiteRepository;

@CapacitorPlugin(
    name = "infa-bgloc",
    permissions = {
        @Permission(
            alias = "location",
            strings = {
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.ACCESS_COARSE_LOCATION,
                Manifest.permission.ACCESS_BACKGROUND_LOCATION,
                Manifest.permission.FOREGROUND_SERVICE
            }
        )
    }
)
public class BgLoc extends Plugin {
    /**
     * Startet den Hintergrunddienst zur Stützpunktdatenerfassung. Die Stützpunktdaten werden dabei
     * in eine lokale SQLite-Datenbank geschrieben und dort aufbewahrt, bis sie (bei Bedarf) an den
     * Server übermittelt oder von der App abgeholt worden sind.
     */
    @PluginMethod()
    public void start(PluginCall call) {
        try {
            if (getPermissionState("location") != PermissionState.GRANTED) {
                requestPermissionForAlias("location", call, "locationPermissionsCallback");
            } else {
                createAndRegisterReceiver(getContext());
                startLocationService(call);
                if (call.hasOption("httpUrl")) {
                    startHttpService(call, false);
                }
                call.resolve();
            }
        } catch (ServiceConfigurationException ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_start_configuration_exception_msg));
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_start_unhandled_exception_msg));
        }
    }

    @PermissionCallback
    private void locationPermissionsCallback(PluginCall call) {
        if (getPermissionState("location") == PermissionState.GRANTED) {
            createAndRegisterReceiver(getContext());
            startLocationService(call);
            if (call.hasOption("httpUrl")) {
                startHttpService(call, false);
            }
            call.resolve();
        } else {
            call.reject(getContext().getString(R.string.location_service_start_missing_permissions));
        }
    }

    @PluginMethod()
    public void requestPermissions(PluginCall call) {
        if (getPermissionState("location") != PermissionState.GRANTED) {
            requestPermissionForAlias("location", call, "locationPermissionsCallback");
        }
        call.resolve();
    }

    @PluginMethod()
    public void checkPermissions(PluginCall call) {
        JSObject ret = new JSObject();
        if (getPermissionState("location") == PermissionState.GRANTED) {
            ret.put("state", "granted"); // matches @capacitor/core -> PermissionResult
        } else {
            ret.put("state", "denied");
        }
        call.resolve(ret);
    }

    @PluginMethod()
    public void upload(PluginCall call) {
        try {
            createAndRegisterReceiver(getContext());
            startHttpService(call, true);
        } catch (ServiceConfigurationException ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_start_configuration_exception_msg));
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_start_unhandled_exception_msg));
        }
    }

    @PluginMethod()
    public void stop(PluginCall call) {
        try {
            stopLocationService();
            stopHttpService();
            call.resolve();
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_stop_unhandled_exception_msg));
        }
    }

    @PluginMethod()
    public void getStoredLocationsCount(PluginCall call) {
        try {
            final SQLiteGetLocationsOptions options = new SQLiteGetLocationsOptions();
            options.setLimit(null);
            options.setSessionToken(null);
            options.setRecipient(SQLiteGetLocationsRecipient.API);

            final SQLiteJSObjectReader reader = new SQLiteJSObjectReader();
            int count = reader.getCount(getContext(), options);
            JSObject ret = new JSObject();
            ret.put("count", count);
            call.resolve(ret);
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_get_stored_locations_count_unhandled_exception_msg));
        }
    }

    @PluginMethod()
    public void getLocations(PluginCall call) {
        try {
            final SQLiteGetLocationsOptions options = new SQLiteGetLocationsOptions();
            if (call.hasOption("sessionToken")) {
                options.setSessionToken(call.getString("sessionToken"));
            } else {
                call.reject(String.format("Missing parameter '%s'.", "sessionToken"));
                return;
            }
            if (call.hasOption("limit")) {
                options.setLimit(call.getInt("limit"));
            }
            if (call.hasOption("ignoreTransmissionFlag")) {
                options.setIgnoreTransmissionFlag(call.getBoolean("ignoreTransmissionFlag"));
            }
            options.setRecipient(SQLiteGetLocationsRecipient.APP);

            final SQLiteJSObjectReader reader = new SQLiteJSObjectReader();
            JSArray arr = new JSArray(reader.getList(getContext(), options).toArray());
            JSObject ret = new JSObject();
            ret.put("items", arr);
            call.resolve(ret);
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_get_locations_unhandled_exception_msg));
        }
    }

    @PluginMethod()
    public void confirmReception(PluginCall call) {
        try {
            final SQLiteFlagOptions options = new SQLiteFlagOptions();
            if (call.hasOption("sessionToken")) {
                options.setSessionToken(call.getString("sessionToken"));
            } else {
                call.reject(String.format("Missing parameter '%s'.", "sessionToken"));
                return;
            }
            if (call.hasOption("ids")) {
                options.setIds(call.getArray("ids").<Integer>toList());
            } else {
                call.reject(String.format("Missing parameter '%s'.", "ids"));
            }
            options.setAppTransmissionFlag(true);

            final SQLiteRepository repository = new SQLiteRepository();
            repository.setFlag(getContext(), options);
            call.resolve();
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_confirm_reception_unhandled_exception_msg));
        }
    }

    @PluginMethod()
    public void cleanup(PluginCall call) {
        try {
            // Identifizieren der Aufrufparameter nach Pflichtfeldern der jeweiligen Optionen
            if (call.hasOption("ids")) {
                cleanupIds(call);
            } else if (call.hasOption("sessionToken")) {
                cleanupSession(call);
            } else {
                final SQLiteRepository repository = new SQLiteRepository();
                repository.cleanup(getContext());
                call.resolve();
            }
        } catch (InvalidOperationException ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_cleanup_session_exception_msg2));
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_cleanup_session_unhandled_exception_msg));
        }
    }

    private void cleanupSession(PluginCall call) {
        try {
            final SQLiteCleanupSessionOptions options = new SQLiteCleanupSessionOptions();
            options.setSessionToken(call.getString("sessionToken"));
            if (call.hasOption("minSessionAge")) {
                options.setMinSessionAge(call.getInt("minSessionAge"));
            }
            if (call.hasOption("forceCleanup")) {
                options.setForceCleanup(call.getBoolean("forceCleanup"));
            }
            final SQLiteRepository repository = new SQLiteRepository();
            repository.cleanupSession(getContext(), options);
            call.resolve();
        } catch (InvalidOperationException ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_cleanup_session_exception_msg2));
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_cleanup_session_unhandled_exception_msg));
        }
    }

    private void cleanupIds(PluginCall call) {
        try {
            final SQLiteCleanupIdsOptions options = new SQLiteCleanupIdsOptions();
            int[] ids = ArrayUtils.toPrimitiveArray(call.getArray("ids").toList());
            options.setIds(ids);
            final SQLiteRepository repository = new SQLiteRepository();
            repository.cleanupIds(getContext(), options);
            call.resolve();
        } catch (InvalidOperationException ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_cleanup_session_exception_msg2));
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_cleanup_session_unhandled_exception_msg));
        }
    }

    @PluginMethod()
    public void getAll(PluginCall call) {
        try {
            SQLiteJSObjectReader reader = new SQLiteJSObjectReader();
            JSArray arr = new JSArray(reader.getList(getContext(), null).toArray());
            JSObject ret = new JSObject();
            ret.put("items", arr);
            call.resolve(ret);
        } catch (Exception ex) {
            rejectByException(call, ex, getContext().getString(R.string.location_service_get_all_unhandled_exception_msg));
        }
    }

    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    private void createAndRegisterReceiver(@NotNull final Context context) {
        //region Private methods
        PluginReceiver mReceiver = new PluginReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                String action = intent.getAction();
                JSObject json;
                switch (Objects.requireNonNull(action)) {
                    case LocationService.INTENT_LOCATION_UPDATE:
                        json = getLocationUpdateResponse(intent);
                        if (json != null) {
                            notifyListeners("onLocationChanged", json);
                        }
                        break;
                    case LocationService.INTENT_LOCATION_IDLE_TIME_EXCEEDED:
                        json = getIdleTimeExceededResponse(intent);
                        notifyListeners("onIdleTimeExceeded", json);
                        break;
                }
            }
        };
        IntentFilter filter = new IntentFilter();
        filter.addAction(LocationService.INTENT_LOCATION_UPDATE);
        filter.addAction(LocationService.INTENT_LOCATION_IDLE_TIME_EXCEEDED);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            context.registerReceiver(mReceiver, filter, Context.RECEIVER_NOT_EXPORTED);
        } else {
            context.registerReceiver(mReceiver, filter);
        }
    }

    private void startLocationService(PluginCall call) {
        PluginConfigurationReader reader = new PluginConfigurationReader();
        ServiceConfiguration serviceConfiguration = reader.getServiceConfiguration(getContext(), call);
        SessionConfiguration sessionConfiguration = reader.getSessionConfiguration(getContext(), call);

        Activity activity = getActivity();
        Intent intent = new Intent(activity, LocationService.class);
        intent.setAction(LocationService.INTENT_START_SERVICE);

        LocationRequest request = serviceConfiguration.toLocationRequest();
        intent.putExtra(LocationService.INTENT_START_EXTRA_LOCATION_REQUEST, request);
        intent.putExtra(LocationService.INTENT_START_EXTRA_SESSION_CONFIG, sessionConfiguration);
        activity.startService(intent);
    }

    private void stopLocationService() {
        Activity activity = getActivity();
        Intent intent = new Intent(activity, LocationService.class);
        intent.setAction(LocationService.INTENT_STOP_SERVICE);
        activity.startService(intent);
    }

    private void startHttpService(PluginCall call, boolean stopWhenFinished) {
        PluginConfigurationReader reader = new PluginConfigurationReader();
        SessionConfiguration sessionConfiguration = reader.getSessionConfiguration(getContext(), call);

        if (sessionConfiguration.getHttpUrl() == null) {
            call.reject(String.format("Missing parameter '%s'.", "httpUrl"));
        }
        Activity activity = getActivity();
        Intent intent = new Intent(activity, HttpService.class);
        intent.setAction(HttpService.INTENT_START_SERVICE);
        intent.putExtra(HttpService.INTENT_START_EXTRA_STOP_WHEN_FINISHED, stopWhenFinished);
        intent.putExtra(HttpService.INTENT_START_EXTRA_SESSION_CONFIG, sessionConfiguration);
        activity.startService(intent);
    }

    private void stopHttpService() {
        Activity activity = getActivity();
        Intent intent = new Intent(activity, HttpService.class);
        intent.setAction(HttpService.INTENT_STOP_SERVICE);
        activity.startService(intent);
    }

    private void rejectByException(PluginCall call, Exception ex, String defaultMessage) {
        String msg = ex.getMessage();
        if (msg == null) {
            msg = defaultMessage;
        }
        Log.e(getAppId(), msg);
        Toast toast = Toast.makeText(getContext(), msg, Toast.LENGTH_LONG);
        toast.show();
        call.reject(msg);
    }
    //endregion
}

