package de.infa.plugins.bgloc.service;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.IBinder;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import androidx.core.app.NotificationCompat;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.toolbox.StringRequest;
import com.android.volley.toolbox.Volley;

import de.infa.plugins.bgloc.R;
import de.infa.plugins.bgloc.service.configuration.SessionConfiguration;
import de.infa.plugins.bgloc.service.exceptions.ServiceConfigurationException;
import de.infa.plugins.bgloc.service.storage.SQLiteFlagOptions;
import de.infa.plugins.bgloc.service.storage.SQLiteGetLocationsOptions;
import de.infa.plugins.bgloc.service.storage.SQLiteGetLocationsRecipient;
import de.infa.plugins.bgloc.service.storage.SQLiteHttpPostTemplateReader;
import de.infa.plugins.bgloc.service.storage.SQLiteJSObjectReader;
import de.infa.plugins.bgloc.service.storage.SQLiteRepository;

public class HttpService extends Service {
  public static final String INTENT_START_SERVICE = "INFA_BGLOC_LOCATION_SERVICE_START";
  public static final String INTENT_STOP_SERVICE = "INFA_BGLOC_LOCATION_SERVICE_STOP";

  public static final String INTENT_START_EXTRA_SESSION_CONFIG = "INFA_BGLOC_INTENT_START_EXTRA_SESSION_CONFIG";
  public static final String INTENT_START_EXTRA_STOP_WHEN_FINISHED = "INFA_BGLOC_INTENT_START_EXTRA_STOP_WHEN_FINISHED";

  @Override
  public IBinder onBind(Intent intent) {
    throw new UnsupportedOperationException(getString(R.string.http_service_bind_exception_msg));
  }

  private SessionConfiguration mSessionConfiguration;
  private Timer mTimer;
  private RequestQueue mRequestQueue;
  private boolean mIsSending;
  private boolean mStopWhenFinished;

  @Override
  public int onStartCommand(Intent intent, int flags, int startId) throws ServiceConfigurationException {
    if (intent != null) {
      String action = intent.getAction();
      if (action != null) {
        switch (action) {
          case INTENT_START_SERVICE:
            mSessionConfiguration = intent.getParcelableExtra(INTENT_START_EXTRA_SESSION_CONFIG);
            if (mSessionConfiguration == null) {
              Log.e(getString(R.string.package_name_short), getString(R.string.location_service_start_missing_session_config_exception_msg));
              throw new ServiceConfigurationException(getString(R.string.location_service_start_missing_session_config_exception_msg));
            } else if (mSessionConfiguration.getSessionToken() == null) {
              Log.e(getString(R.string.package_name_short), getString(R.string.location_service_start_missing_session_token_exception_msg));
              throw new ServiceConfigurationException(getString(R.string.location_service_start_missing_session_token_exception_msg));
            }
            doStartForeground(intent.getBooleanExtra(INTENT_START_EXTRA_STOP_WHEN_FINISHED, false));
            doStartUploading();
            break;
          case INTENT_STOP_SERVICE:
            mStopWhenFinished = true;
            break;
        }
      }
    }
    return START_STICKY;
    //return super.onStartCommand(intent, flags, startId);
  }

  private void doStartForeground(boolean stopWhenFinished) {
    Notification notification = getNotification();
    startForeground(2, notification);
    mStopWhenFinished = stopWhenFinished;
  }

  private void doStopForeground() {
    stopForeground(true);
    stopSelf();
  }

  private int getRecordCount() {
    SQLiteJSObjectReader reader = new SQLiteJSObjectReader();
    SQLiteGetLocationsOptions options = new SQLiteGetLocationsOptions();
    options.setSessionToken(mSessionConfiguration.getSessionToken());
    options.setRecipient(SQLiteGetLocationsRecipient.API);
    options.setLimit(mSessionConfiguration.getHttpPostLimit());
    return reader.getCount(HttpService.this, options);
  }

  private void doStartUploading() {
    if (mTimer != null && mRequestQueue != null) {
      doStopUploading();
    }
    mRequestQueue = Volley.newRequestQueue(HttpService.this);
    mTimer = new Timer();
    mTimer.scheduleAtFixedRate(new TimerTask() {
      @Override
      public void run() {
        if (mIsSending) {
          return;
        }

        int count = getRecordCount();
        if (count == 0) {
          if (mStopWhenFinished) {
            doStopUploading();
            doStopForeground();
          }
          return;
        }

        SQLiteJSObjectReader reader = new SQLiteJSObjectReader();
        boolean hasEntries = reader.hasEntries(HttpService.this, mSessionConfiguration.getSessionToken(), SQLiteGetLocationsRecipient.API);
        if (!hasEntries) {
          return;
        }

        Log.v(HttpService.this.getString(R.string.package_name_short), String.format("%d new location(s) found within storage. Attempting to upload...", count));
        mIsSending = true;
        String url = mSessionConfiguration.getHttpUrl();
        StringRequest request = new StringRequest(Request.Method.POST, url,
          response -> {
            try {
              JSONArray json = new JSONArray(response);
              ArrayList<Integer> ids = new ArrayList<>();
              for (int i = 0; i < json.length(); i++) {
                ids.add(json.getInt(i));
              }
              SQLiteFlagOptions options = new SQLiteFlagOptions();
              options.setSessionToken(mSessionConfiguration.getSessionToken());
              options.setIds(ids);
              options.setApiTransmissionFlag(true);
              SQLiteRepository recorder = new SQLiteRepository();
              recorder.setFlag(HttpService.this, options);
            } catch (JSONException ex) {
              Log.e(getString(R.string.package_name_short), "Cannot evaluate server response.", ex);
              mIsSending = false;
            } finally {
              mIsSending = false;
            }
          },
          error -> {
            if (error.networkResponse == null) {
              Log.e(getString(R.string.package_name_short), "Server not available or responding.", error);
            } else {
              String networkResponse = error.networkResponse.data != null ? new String(error.networkResponse.data) : "";
              Log.e(getString(R.string.package_name_short), String.format("Cannot send recorded positions to server.\r\n%s", networkResponse), error);
            }
            mIsSending = false;
          })
        {
          @Override
          public Map<String, String> getHeaders() {
            HashMap<String, String> headers = new HashMap<>();
            if (mSessionConfiguration.getHttpAuthentication() != null) {
              headers.put("Authorization", mSessionConfiguration.getHttpAuthentication());
            }
            return headers;
          }

          @Override
          public String getBodyContentType() {
            return "application/json";
          }

          @Override
          public byte[] getBody() {
            SQLiteGetLocationsOptions options = new SQLiteGetLocationsOptions();
            options.setSessionToken(mSessionConfiguration.getSessionToken());
            options.setRecipient(SQLiteGetLocationsRecipient.API);
            options.setLimit(mSessionConfiguration.getHttpPostLimit());
            SQLiteHttpPostTemplateReader reader = new SQLiteHttpPostTemplateReader() {
              @Override
              public String getHttpPostTemplate() {
                return mSessionConfiguration.getHttpPostTemplate();
              }
            };
            ArrayList<JSONObject> list = reader.getList(HttpService.this, options);
            JSONArray json = new JSONArray();
            for (JSONObject x : list) {
              json.put(x);
            }
            return json.toString().getBytes();
          }
        };
        mRequestQueue.add(request);
      }
    }, 0, mSessionConfiguration.getHttpPostFrequency());
  }

  private void doStopUploading() {
    mTimer.cancel();
    mTimer = null;
    mRequestQueue.stop();
    mRequestQueue = null;
  }

  private Notification getNotification() {
    Intent intent = new Intent();
    PendingIntent pendingIntent = PendingIntent.getActivity(HttpService.this, 0, intent, PendingIntent.FLAG_IMMUTABLE);

    NotificationCompat.Builder builder;
    NotificationChannel channel = new NotificationChannel(
            String.format("%s_CHANNEL_ID", this.getPackageName().toUpperCase()),
            String.format("%s Service notification channel", getString(R.string.package_name_short)), NotificationManager.IMPORTANCE_HIGH);
    NotificationManager notificationManager = (NotificationManager) this.getSystemService(Context.NOTIFICATION_SERVICE);
    if (notificationManager != null) {
      notificationManager.createNotificationChannel(channel);
    }
    builder = new NotificationCompat.Builder(HttpService.this, channel.getId());

    NotificationCompat.BigTextStyle bigTextStyle = new NotificationCompat.BigTextStyle();
    bigTextStyle.setBigContentTitle(getString(R.string.package_name));
    bigTextStyle.bigText(getString(R.string.http_service_notification_bigtext));
    builder.setStyle(bigTextStyle);

    builder.setWhen(System.currentTimeMillis());
    builder.setSmallIcon(R.drawable.ic_baseline_cloud_24_w80);
    Bitmap largeIconBitmap = BitmapFactory.decodeResource(this.getResources(), R.drawable.ic_baseline_cloud_24_w80);
    builder.setLargeIcon(largeIconBitmap);

    builder.setPriority(Notification.PRIORITY_MAX);

    builder.setFullScreenIntent(pendingIntent, true);
    return builder.build();
  }
}
