package de.infa.plugins.bgloc.service.storage;

import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.location.Location;
import android.os.Bundle;
import android.util.Log;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.File;
import java.time.Instant;
import java.util.ArrayList;
import java.util.Arrays;

import de.infa.plugins.bgloc.R;
import de.infa.plugins.bgloc.service.exceptions.InvalidOperationException;

public class SQLiteRepository {
  protected static final String TABLE_NAME = "locations";
  protected static final String INTENT_BROADCAST_DB_LOCATION_STORED = "INFA_BGLOC_INTENT_BROADCAST_DB_LOCATION_STORED";

  protected SQLiteDatabase getDatabase(@NotNull final Context context) {
    File file = new File(context.getFilesDir(), "infa-bgloc.db");
    return SQLiteDatabase.openOrCreateDatabase(file, null);
  }

  protected boolean tableExists(@NotNull final SQLiteDatabase db) {
    try (Cursor cursor = db.rawQuery(String.format("select name from sqlite_master where name = '%s'", TABLE_NAME), null)) {
      return cursor.getCount() > 0;
    }
  }

  public int getCount(@NotNull final Context context) {
    try (SQLiteDatabase db = getDatabase(context)) {
      if (tableExists(db)) {
        try (Cursor cursor = db.rawQuery("select count(id) from " + TABLE_NAME, null)) {
          cursor.moveToFirst();
          if (!cursor.isAfterLast()) {
            return cursor.getInt(0);
          }
        }
      }
      return 0;
    }
  }

  public int getCount(@NotNull final Context context, @Nullable SQLiteGetLocationsOptions options) {
    try (SQLiteDatabase db = getDatabase(context)) {
      if (tableExists(db)) {
        try (Cursor cursor = db.query(TABLE_NAME,
          new String[]{"id", "session_token", "api_transmission_flag", "app_transmission_flag",
            "time", "latitude", "longitude", "altitude", "speed", "bearing",
            "h_acc", "v_acc", "speed_acc", "bearing_acc", "nanos", "provider"},
          options != null ? options.getQuerySelection() : null,
          options != null ? options.getQuerySelectionArgs() : null,
          null, null, "id",
          options != null ? options.getLimitArgs() : null
        )) {
          return cursor.getCount();
        }
      }
      return 0;
    }
  }

  public int put(@NotNull final Context context, @NotNull final Location location, final String sessionToken,
                 final Boolean apiSendFlag, final Boolean appSendFlag) {
    try (SQLiteDatabase db = getDatabase(context)) {
      db.execSQL(String.format(
        "create table if not exists %s (" +
        "id integer primary key autoincrement, " +
        "session_token text, " +
        "api_transmission_flag int, " +
        "app_transmission_flag int, " +
        "time integer not null, " +
        "latitude real not null, " +
        "longitude real not null, " +
        "altitude real, " +
        "speed real, " +
        "bearing real, " +
        "h_acc real, " +
        "v_acc real, " +
        "speed_acc real, " +
        "bearing_acc real, " +
        "nanos integer not null, " +
        "provider text)", TABLE_NAME));
      try {
        Bundle extras = location.getExtras();
        ContentValues cv = new ContentValues();
        if (sessionToken != null) {
          cv.put("session_token", sessionToken);
        }
        cv.put("api_transmission_flag", apiSendFlag);
        cv.put("app_transmission_flag", appSendFlag);
        cv.put("time", location.getTime());
        // write smoothened values if present
        cv.put("latitude", location.getLatitude());
        cv.put("longitude", location.getLongitude());
        if (location.hasAltitude()) {
          cv.put("altitude", location.getAltitude());
        }
        if (location.hasSpeed()) {
          cv.put("speed", location.getSpeed());
        }
        if (location.hasBearing()) {
          cv.put("bearing", location.getBearing());
        }
        if (location.hasAccuracy()) {
          cv.put("h_acc", location.getAccuracy());
        }
        if (location.hasVerticalAccuracy()) {
          cv.put("v_acc", location.getVerticalAccuracyMeters());
        }
        if (location.hasSpeedAccuracy()) {
          cv.put("speed_acc", location.getSpeedAccuracyMetersPerSecond());
        }
        if (location.hasBearingAccuracy()) {
          cv.put("bearing_acc", location.getBearingAccuracyDegrees());
        }
        cv.put("nanos", location.getElapsedRealtimeNanos());
        cv.put("provider", location.getProvider());
        if (db.insertOrThrow(TABLE_NAME, "", cv) > -1) {
          try (Cursor cursor = db.rawQuery("select max(id) from " + TABLE_NAME, new String[0])) {
            cursor.moveToFirst();
            if (!cursor.isAfterLast()) {
              context.sendBroadcast(new Intent(INTENT_BROADCAST_DB_LOCATION_STORED));
              return cursor.getInt(0);
            }
          }
        }
        return -1;
      } catch (Exception ex) {
        Log.e("INFA-BgLoc", String.format("Storing location(s) to local SQLite database failed. %s", ex.getMessage()));
        return -1;
      }
    }
  }

  public void setFlag(@NotNull final Context context, final SQLiteFlagOptions options) {
    ArrayList<String> selection = new ArrayList<>();
    ArrayList<String> selectionArgs = new ArrayList<>();
    if (options.getSessionToken() != null) {
      selection.add("session_token = ?");
      selectionArgs.add(options.getSessionToken());
    }
    if (!options.getIds().isEmpty()) {
      String[] ids = options.getIds().stream().map(i -> Integer.toString(i)).toArray(String[]::new);
      selection.add(String.format("id in (%s)", String.join(", ", ids)));
    }
    if (selection.size() == 0) {
      throw new IllegalArgumentException("Cannot run setFlag() without any options.");
    }

    try (SQLiteDatabase db = getDatabase(context)) {
      ContentValues cv = new ContentValues();
      if (options.hasApiTransmissionFlag()) {
        cv.put("api_transmission_flag", options.getApiTransmissionFlag());
      }
      if (options.hasAppTransmissionFlag()) {
        cv.put("app_transmission_flag", options.getAppTransmissionFlag());
      }
      try {
        db.update(TABLE_NAME, cv, String.join(" and ", selection), selectionArgs.toArray(new String[0]));
      } catch (Exception ex) {
        Log.e("INFA-BgLoc", String.format("Updating flags to local SQLite database failed.", ex.getMessage()));
      }
    }
  }

  // Not in use and turned into comment
//  protected long getSessionAge(@NotNull final Context context, @NotNull final SQLiteDatabase db, @NotNull final String sessionToken) {
//    try (Cursor cursor = db.rawQuery("select max(time) from " + TABLE_NAME + " where session_token = ?", new String[] { sessionToken }))
//    {
//      cursor.moveToFirst();
//      if (!cursor.isAfterLast()) {
//        long gpsTimeEpochMs = cursor.getLong(0);
//        long currentTimeEpochMs = Instant.now().toEpochMilli();
//        return currentTimeEpochMs - gpsTimeEpochMs;
//      }
//      throw new NotFoundException(String.format(context.getString(R.string.location_service_session_not_found_exception_msg), sessionToken));
//    }
//  }

  protected boolean canCleanupSession(@NotNull final Context context,
                                    @NotNull final SQLiteDatabase db,
                                    @NotNull final SQLiteCleanupSessionOptions options) throws IllegalArgumentException {
    String sessionToken = options.getSessionToken();
    if (sessionToken == null) {
      throw new IllegalArgumentException(context.getString(R.string.location_service_start_missing_session_token_exception_msg));
    }

    if (options.getForceCleanup()) {
      return true;
    }

    if (!this.tableExists(db)) {
      return false;
    }

    try (Cursor cursor = db.rawQuery("select count(id) from " + TABLE_NAME +
      " where (api_transmission_flag = 0 or app_transmission_flag = 0) and session_token = ?", new String[] { sessionToken })) {
      cursor.moveToFirst();
      if (cursor.getInt(0) > 0) {
        return false;
      }
    }

    if (options.getMinSessionAge() != null) {
      try (Cursor cursor = db.rawQuery("select max(time) from " + TABLE_NAME +
        " where session_token = ?", new String[] { sessionToken }))
      {
        cursor.moveToFirst();
        if (cursor.isAfterLast()) {
          final long gpsTimeEpochMs = cursor.getLong(0);
          if (gpsTimeEpochMs + options.getMinSessionAge() > Instant.now().toEpochMilli()) {
            return false;
          }
        }
      }
    }
    return true;
  }

  public void cleanup(@NotNull final Context context) {
    try (SQLiteDatabase db = getDatabase(context)) {
      if (this.tableExists(db)) {
        db.delete(TABLE_NAME, null, null);
      }
    }
  }

  public void cleanupSession(@NotNull final Context context, final SQLiteCleanupSessionOptions options) {
    try (SQLiteDatabase db = getDatabase(context)) {
      if (canCleanupSession(context, db, options)) {
        int delete = db.delete(TABLE_NAME, "session_token = ?", new String[] { options.getSessionToken() });
      } else {
        throw new InvalidOperationException(String.format("%s\r\n%s",
          context.getString(R.string.location_service_cleanup_session_exception_msg2),
          context.getString(R.string.location_service_cleanup_session_exception_hint)));
      }
    }
  }

  public void cleanupIds(@NotNull final Context context, final SQLiteCleanupIdsOptions options) {
    try (SQLiteDatabase db = getDatabase(context)) {
      if (this.tableExists(db)) {
        String arg = Arrays.stream(options.getIds()).mapToObj(String::valueOf).reduce((a, b) -> a.concat(",").concat(b)).get();
        String[] args = {arg};
        int delete = db.delete(TABLE_NAME, "id in (?)", args);
      }
    }
  }
}
