//@ts-ignore
import { BehaviorSubject, Subject } from 'rxjs';
import { InfaBgLocEvent, } from './models/interfaces';
import BgLoc from './definitions';
export class InfaBgLocService {
    constructor() {
        this.timer = undefined;
        this.waitingForResponse = false;
        this.stopFlag = false;
        this.active = new BehaviorSubject(false);
        this.location = new Subject();
        this.idleTimeout = new Subject();
        if (!BgLoc) {
            throw new Error(`Plugin not found (BgLoc).`);
        }
        BgLoc.addListener('onLocationChanged', (err) => {
            this.location.next(err);
        });
        BgLoc.addListener('onIdleTimeExceeded', (err) => {
            this.idleTimeout.next(err);
        });
    }
    /** Start the native background-location service by using the parameterized configuration.
     * Define a handler, if you want to process the locations within your app safely. The
     * returned numbers will then be confirmed to the native plugin.
     */
    async start(config, handler, limit) {
        if (this.active.getValue()) {
            await this.doStop();
        }
        this.handler = handler;
        // If not explicitly set, the flag is set depending on a customized handler (handler = true, no handler = false)
        if (config.waitForClientCollection === undefined) {
            config.waitForClientCollection = !!handler;
        }
        return BgLoc.start(config).then(() => {
            console.debug(`BgLoc service started.`);
            this.timer = setInterval(async (config) => {
                if (this.handler) {
                    if (this.waitingForResponse) {
                        return;
                    }
                    this.waitingForResponse = true;
                    try {
                        let locations = await BgLoc.getLocations({
                            sessionToken: config.sessionToken,
                            limit: limit ? limit : 100,
                        });
                        if (locations.items.length > 0) {
                            let ids = await this.handler(locations);
                            await BgLoc.confirmReception({
                                sessionToken: config.sessionToken,
                                ids: ids,
                            });
                        }
                    }
                    catch (ex) {
                        console.error(`Cannot handle locations from plugin (BgLoc).`, ex);
                    }
                    finally {
                        this.waitingForResponse = false;
                    }
                }
                if (this.stopFlag) {
                    await this.doStop();
                }
            }, config.interval, config);
            this.stopFlag = false;
            this.active.next(true);
        }, ex => {
            console.error(`Cannot start service (BgLoc).`, ex);
            return Promise.reject(ex);
        });
    }
    /** Stop the native background-location service. */
    stop() {
        this.stopFlag = true;
    }
    doStop() {
        return BgLoc.stop().then(() => {
            console.debug(`BgLoc service stopped.`);
            clearInterval(this.timer);
            this.timer = null;
            this.stopFlag = false;
            this.active.next(false);
        }, ex => {
            console.error(`Cannot stop service (BgLoc).`, ex);
            return Promise.reject(ex);
        });
    }
    /** Clean up the plugin repository. Only succeeds, if all locations
     * have been transmitted already or if you force the cleanup through the
     * parameter for a specific session.
     */
    cleanup(options) {
        if (options) {
            return BgLoc.cleanup(options).then(() => console.debug(`Storage cleaned up.`, options), ex => {
                console.error(`Cleanup failed.`, options, ex);
                return Promise.reject(ex);
            });
        }
        else {
            return BgLoc.cleanup().then(() => console.debug(`Entire storage has been cleaned up.`), ex => {
                console.error(`Cleaning up entire storage failed.`, ex);
                return Promise.reject(ex);
            });
        }
    }
    /** Start the http service without starting the location service itself.
     * When every location has been uploaded to the server, the service
     * shuts down.
     */
    upload(config) {
        return BgLoc.upload(config).then(() => console.debug(`Uploading locations for session ${config.sessionToken}.`, config), ex => {
            console.error(`Cannot upload locations for session ${config.sessionToken}.`, ex);
            return Promise.reject(ex);
        });
    }
    /** If options are set, returns the unconfirmed locations of the given session.
     * Otherwise, returns all locations still stored in the repository.
     */
    get(options) {
        if (options) {
            return BgLoc.getLocations(options).then((r) => r, ex => {
                console.error(`Cannot read locations from plugin for session ${options.sessionToken}.`, options, ex);
                return Promise.reject(ex);
            });
        }
        else {
            return BgLoc.getAll().then((r) => r, ex => {
                console.error(`Cannot read locations from plugin (BgLoc).`, options, ex);
                return Promise.reject(ex);
            });
        }
    }
    getStoredLocationsCount() {
        return BgLoc.getStoredLocationsCount().then((r) => Promise.resolve(r.count), ex => {
            console.error(`Cannot read stored location count from plugin.`, ex);
            return Promise.reject(ex);
        });
    }
    /** Confirm the locations to be received by the app. Otherwise, they
     * will be returned again with the next get() request.
     */
    confirm(options) {
        return BgLoc.confirmReception(options);
    }
    /** Provides an event based update mechanism. */
    on(event) {
        switch (event) {
            case InfaBgLocEvent.LOCATION:
                return this.location;
            case InfaBgLocEvent.TIMEOUT:
                return this.idleTimeout;
        }
    }
    /** Requests the required permissions, just in case, that they are not granted already. */
    async requestPermissions() {
        await BgLoc.requestPermissions();
    }
    /** Answers true, when all required permissions are already granted. */
    async checkPermissions() {
        const status = await BgLoc.checkPermissions();
        return status.location === 'granted';
    }
}
//# sourceMappingURL=infa-bgloc.service.js.map