export declare enum InfaBgLocEvent {
    LOCATION = "location",
    TIMEOUT = "timeout"
}
export declare enum InfaBgLocPriority {
    HIGH_ACCURACY = 100,
    BALANCED_POWER_ACCURACY = 102,
    LOW_POWER = 104,
    NO_POWER = 105
}
export interface InfaBgLocSessionConfig {
    sessionToken: string;
    /** If set, the service will post the recorded locations to the given server url end-point as soon
     * as there is a valid internet connection available. The data will be sent as POST-Method and
     * (actually) without any authorization header. Therefore the end-point should allow anonymous access.
     * @default null
     */
    httpUrl?: string;
    /** Defines the JSON-based template used for sending a location to the server.
     * Please use the following wildcards as desired:
     * - %id% (the auto-incremental identifier for every recorded location)
     * - %sessionToken% (the identifier used for the particular recording session)
     * - %apiSendStatus% (retrieves 0 or 1, whether the location has been flagged to already be sent to the server)
     * - %appSendStatus% (retrieves 0 or 1, whether the location has been flagged to already be collected by the app itself)
     * - %time% (the location's UTC time fix, in milliseconds since epoch (January 1, 1970).)
     * - %latitude% (the latitude, in degrees)
     * - %longitude% (the longitude, in degrees)
     * - %altitude% (the altitude if available, in meters above the WGS 84 reference ellipsoid.)
     * - %speed% (the speed if it is available, in meters/second over ground.)
     * - %bearing% (the bearing, in degrees [0..360])
     * - %horizontalAccuracy% (the estimated horizontal accuracy of this location, radial, in meters.)
     * - %verticalAccuracy% (the estimated vertical accuracy of this location, in meters.)
     * - %speedAccuracy% (the estimated speed accuracy of this location, in meters per second.)
     * - %bearingAccuracy% (the estimated bearing accuracy of this location, in degrees.)
     * - %nanos% (the time of this fix, in elapsed real-time since system boot.)
     * - %provider% (the name of the provider that generated this fix.)
     * @example
     * {
     *   "id": %id%,
     *   "session": %sessionToken%,
     *   "lat": %latitude%,
     *   "lng": %longitude%,
     *   "alt": %altitude%,
     *   "time": %time%,
     *   "user": LOCAL_USER_VARIABLE
     * }
     */
    httpPostTemplate?: string;
    httpAuthentication?: string;
    /** Defines the number of locations maximal sent to the server at once.
     * @default MAX_INTEGER
    */
    httpPostLimit?: number;
    /** Defines the server upload frequency in milliseconds.
     * @default 10000
    */
    httpPostFrequency?: number;
    /** When the location has not changed during the given amount of milliseconds,
     * the service returns a notification. The default time is 4620000
     */
    idleTimeUntilNotification?: number;
    /** Indicates, whether a location shall wait for the client to be collected,
     * before it can be (regularly) cleared from the storage.
     */
    waitForClientCollection?: boolean;
    /** Allows the service to persist in the background, although the main
     * app itself might have been shut down already.
     * @default false
     */
    backgroundService?: boolean;
}
export interface InfaBgLocGeolocationConfig {
    /** Set the desired interval of a location request, in milliseconds. */
    interval: number;
    /** Set the fastest interval of a location request, in milliseconds. */
    fastestInterval?: number;
    /** Set the minimum displacement between location updates in meters.
     * @default null
    */
    smallestDisplacement?: number;
    /** Set the quality of the request.
     * - HIGH_ACCURACY = 100
     * - BALANCED_POWER_ACCURACY = 102
     * - LOW_POWER = 104
     * - NO_POWER = 105
     * @default 100
     */
    priority?: number;
    /** Set the request expiration time, in milliseconds.
     * @default null
    */
    expiration?: number;
    /** Set the unique token for the current recording session. (required) */
    sessionToken: string;
}
/** Used to configure the background service */
export interface InfaBgLocConfig extends InfaBgLocGeolocationConfig, InfaBgLocSessionConfig {
}
export interface InfaBgLocFetchOptions {
    /** Set the session to retrieve the locations for. (required) */
    sessionToken: string;
    /** Limits the number of returned locations and retrieves the oldest entries.
     * @default null */
    limit?: number;
    /** Get the entire dataset for this session, regardless of the current app-transmission flag.
     * @default false */
    ignoreTransmissionFlag?: boolean;
}
export interface InfaBgLocConfirmReceptionOptions {
    /** Set the session to confirm the reception */
    sessionToken: string;
    /** Set the array of ids handled within the application to
     * mark all locations within the native plugin as handled up to this one.
     */
    ids: number[];
}
export interface InfaBgLocCleanupSessionOptions {
    /** The session to be cleaned up. (required) */
    sessionToken: string;
    /** Defines the minimum age of the session in days, before it can be discarded.
     * A value of 0 allows the service to immediately delete the location.
     * @default 0
    */
    minSessionAge?: number;
    /** All locations have to be posted to a server (through HTTP-POST) by the native
     * upload client, before any location of the session will be deleted.
     * @default true
     */
    forceCleanup?: boolean;
}
export interface InfaBgLocCleanupByIdsOptions {
    /** The ids to be cleaned up. */
    ids: number[];
}
export interface InfaBgLocation {
    id: number;
    sessionToken: string;
    time: number;
    latitude: number;
    longitude: number;
    altitude?: number;
    speed?: number;
    bearing?: number;
    angularSpeed?: number;
    accuracy?: number;
    verticalAccuracy?: number;
    speedAccuracy?: number;
    bearingAccuracy?: number;
    angularSpeedAccuracy?: number;
    nanos: number;
    provider: string;
    latitudeRaw?: number;
    longitudeRaw?: number;
}
export interface InfaBgLocationArray {
    items: InfaBgLocation[];
}
export interface InfaBgLocationCount {
    count: number;
}
export interface InfaBgLocIdleTimeExceededResponse {
    /** The session token of the pending record. */
    sessionToken: string;
    /** The reached time limit once set up for this record. */
    timeExceeded: number;
}
