var capacitorBgLoc = (function (exports, core, rxjs) {
    'use strict';

    exports.InfaBgLocEvent = void 0;
    (function (InfaBgLocEvent) {
        InfaBgLocEvent["LOCATION"] = "location";
        InfaBgLocEvent["TIMEOUT"] = "timeout";
    })(exports.InfaBgLocEvent || (exports.InfaBgLocEvent = {}));
    exports.InfaBgLocPriority = void 0;
    (function (InfaBgLocPriority) {
        InfaBgLocPriority[InfaBgLocPriority["HIGH_ACCURACY"] = 100] = "HIGH_ACCURACY";
        InfaBgLocPriority[InfaBgLocPriority["BALANCED_POWER_ACCURACY"] = 102] = "BALANCED_POWER_ACCURACY";
        InfaBgLocPriority[InfaBgLocPriority["LOW_POWER"] = 104] = "LOW_POWER";
        InfaBgLocPriority[InfaBgLocPriority["NO_POWER"] = 105] = "NO_POWER";
    })(exports.InfaBgLocPriority || (exports.InfaBgLocPriority = {}));

    const BgLoc = core.registerPlugin('infa-bgloc', {
        web: () => Promise.resolve().then(function () { return web; }).then(m => new m.BgLocWeb()),
    });

    class BgLocWeb extends core.WebPlugin {
        checkPermissions() {
            return Promise.resolve({ location: 'granted' });
        }
        requestPermissions() {
            return Promise.resolve({ location: 'granted' });
        }
        start(config) {
            if (!BgLocWeb.timer) {
                BgLocWeb.timer = setInterval(() => {
                    navigator.geolocation.getCurrentPosition(position => {
                        let location = {
                            id: BgLocWeb.lastId++,
                            sessionToken: config.sessionToken,
                            time: position.timestamp,
                            latitude: position.coords.latitude,
                            longitude: position.coords.longitude,
                            altitude: position.coords.altitude
                                ? position.coords.altitude
                                : undefined,
                            speed: position.coords.speed ? position.coords.speed : undefined,
                            bearing: position.coords.heading
                                ? position.coords.heading
                                : undefined,
                            accuracy: position.coords.accuracy
                                ? position.coords.accuracy
                                : 10,
                            provider: 'HTML5',
                            nanos: position.timestamp * 1000,
                        };
                        BgLocWeb.records.push(location);
                        BgLocWeb.location.next(location);
                    }, error => {
                        console.error('ERROR', error);
                    }, {
                        enableHighAccuracy: config.priority === 100,
                    });
                }, 2000);
                return Promise.resolve();
            }
            else {
                return Promise.reject('BgLoc (web) already running.');
            }
        }
        stop() {
            if (!!BgLocWeb.timer) {
                clearInterval(BgLocWeb.timer);
                BgLocWeb.timer = undefined;
                return Promise.resolve();
            }
            else {
                return Promise.reject();
            }
        }
        getLocations(options) {
            let arr = BgLocWeb.records
                .filter(x => x.sessionToken === options.sessionToken)
                .sort((a, b) => a.id - b.id)
                .slice(0, options.limit
                ? Math.min(options.limit, BgLocWeb.records.length) - 1
                : BgLocWeb.records.length - 1);
            return Promise.resolve({
                items: arr,
            });
        }
        getStoredLocationsCount() {
            return Promise.resolve({
                count: BgLocWeb.records.length,
            });
        }
        getAll() {
            return Promise.resolve({
                items: BgLocWeb.records,
            });
        }
        confirmReception(options) {
            for (let id of options.ids) {
                let idx = BgLocWeb.records.findIndex(x => x.sessionToken === options.sessionToken && x.id === id);
                if (idx > -1) {
                    BgLocWeb.records.splice(idx, 1);
                }
            }
            return Promise.resolve();
        }
        upload(config) {
            console.log('BgLoc (web) upload triggered (not supported)', config);
            return Promise.resolve();
        }
        cleanup(options) {
            if (!options) {
                BgLocWeb.records = [];
            }
            else {
                let ids = [];
                if (options.ids) {
                    ids = options.ids;
                }
                else if (options.sessionToken) {
                    ids = BgLocWeb.records
                        .filter(x => x.sessionToken === options.sessionToken)
                        .map(x => x.id);
                }
                while (ids.length > 0) {
                    let idx = BgLocWeb.records.findIndex(x => x.id === ids[0]);
                    BgLocWeb.records.splice(idx, 1);
                    ids.splice(0, 1);
                }
            }
            return Promise.resolve();
        }
        addListener(eventName, listenerFunc) {
            let handle;
            switch (eventName) {
                case 'onLocationChanged':
                    let subscription = BgLocWeb.location.subscribe((x) => {
                        listenerFunc(x); // do use first arg "err" as response
                    });
                    handle = {
                        remove: async () => {
                            await subscription.unsubscribe();
                        },
                    };
                    break;
                case 'onIdleTimeExceeded':
                    console.warn(`Timeout event not supported by web platform.`);
                    handle = {
                        remove: async () => {
                            await subscription.unsubscribe();
                        },
                    };
                    break;
                default:
                    console.warn(`Invalid BgLoc event ${eventName}.`);
                    throw `Invalid BgLoc event ${eventName}.`;
            }
            const result = Promise.resolve(handle);
            result.remove = handle;
            return result;
        }
        on(eventName) {
            switch (eventName) {
                case exports.InfaBgLocEvent.LOCATION:
                    return BgLocWeb.location;
                case exports.InfaBgLocEvent.TIMEOUT:
                    console.warn(`Timeout event not supported by web platform.`);
                    return new rxjs.Observable();
                default:
                    console.warn(`Invalid BgLoc event ${eventName}.`);
                    return new rxjs.Observable();
            }
        }
        async echo(options) {
            console.log('ECHO', options);
            return options;
        }
    }
    BgLocWeb.timer = undefined;
    BgLocWeb.lastId = 1;
    BgLocWeb.records = [];
    BgLocWeb.location = new rxjs.Subject();

    var web = /*#__PURE__*/Object.freeze({
        __proto__: null,
        BgLocWeb: BgLocWeb
    });

    //@ts-ignore
    class InfaBgLocService {
        constructor() {
            this.timer = undefined;
            this.waitingForResponse = false;
            this.stopFlag = false;
            this.active = new rxjs.BehaviorSubject(false);
            this.location = new rxjs.Subject();
            this.idleTimeout = new rxjs.Subject();
            if (!BgLoc) {
                throw new Error(`Plugin not found (BgLoc).`);
            }
            BgLoc.addListener('onLocationChanged', (err) => {
                this.location.next(err);
            });
            BgLoc.addListener('onIdleTimeExceeded', (err) => {
                this.idleTimeout.next(err);
            });
        }
        /** Start the native background-location service by using the parameterized configuration.
         * Define a handler, if you want to process the locations within your app safely. The
         * returned numbers will then be confirmed to the native plugin.
         */
        async start(config, handler, limit) {
            if (this.active.getValue()) {
                await this.doStop();
            }
            this.handler = handler;
            // If not explicitly set, the flag is set depending on a customized handler (handler = true, no handler = false)
            if (config.waitForClientCollection === undefined) {
                config.waitForClientCollection = !!handler;
            }
            return BgLoc.start(config).then(() => {
                console.debug(`BgLoc service started.`);
                this.timer = setInterval(async (config) => {
                    if (this.handler) {
                        if (this.waitingForResponse) {
                            return;
                        }
                        this.waitingForResponse = true;
                        try {
                            let locations = await BgLoc.getLocations({
                                sessionToken: config.sessionToken,
                                limit: limit ? limit : 100,
                            });
                            if (locations.items.length > 0) {
                                let ids = await this.handler(locations);
                                await BgLoc.confirmReception({
                                    sessionToken: config.sessionToken,
                                    ids: ids,
                                });
                            }
                        }
                        catch (ex) {
                            console.error(`Cannot handle locations from plugin (BgLoc).`, ex);
                        }
                        finally {
                            this.waitingForResponse = false;
                        }
                    }
                    if (this.stopFlag) {
                        await this.doStop();
                    }
                }, config.interval, config);
                this.stopFlag = false;
                this.active.next(true);
            }, ex => {
                console.error(`Cannot start service (BgLoc).`, ex);
                return Promise.reject(ex);
            });
        }
        /** Stop the native background-location service. */
        stop() {
            this.stopFlag = true;
        }
        doStop() {
            return BgLoc.stop().then(() => {
                console.debug(`BgLoc service stopped.`);
                clearInterval(this.timer);
                this.timer = null;
                this.stopFlag = false;
                this.active.next(false);
            }, ex => {
                console.error(`Cannot stop service (BgLoc).`, ex);
                return Promise.reject(ex);
            });
        }
        /** Clean up the plugin repository. Only succeeds, if all locations
         * have been transmitted already or if you force the cleanup through the
         * parameter for a specific session.
         */
        cleanup(options) {
            if (options) {
                return BgLoc.cleanup(options).then(() => console.debug(`Storage cleaned up.`, options), ex => {
                    console.error(`Cleanup failed.`, options, ex);
                    return Promise.reject(ex);
                });
            }
            else {
                return BgLoc.cleanup().then(() => console.debug(`Entire storage has been cleaned up.`), ex => {
                    console.error(`Cleaning up entire storage failed.`, ex);
                    return Promise.reject(ex);
                });
            }
        }
        /** Start the http service without starting the location service itself.
         * When every location has been uploaded to the server, the service
         * shuts down.
         */
        upload(config) {
            return BgLoc.upload(config).then(() => console.debug(`Uploading locations for session ${config.sessionToken}.`, config), ex => {
                console.error(`Cannot upload locations for session ${config.sessionToken}.`, ex);
                return Promise.reject(ex);
            });
        }
        /** If options are set, returns the unconfirmed locations of the given session.
         * Otherwise, returns all locations still stored in the repository.
         */
        get(options) {
            if (options) {
                return BgLoc.getLocations(options).then((r) => r, ex => {
                    console.error(`Cannot read locations from plugin for session ${options.sessionToken}.`, options, ex);
                    return Promise.reject(ex);
                });
            }
            else {
                return BgLoc.getAll().then((r) => r, ex => {
                    console.error(`Cannot read locations from plugin (BgLoc).`, options, ex);
                    return Promise.reject(ex);
                });
            }
        }
        getStoredLocationsCount() {
            return BgLoc.getStoredLocationsCount().then((r) => Promise.resolve(r.count), ex => {
                console.error(`Cannot read stored location count from plugin.`, ex);
                return Promise.reject(ex);
            });
        }
        /** Confirm the locations to be received by the app. Otherwise, they
         * will be returned again with the next get() request.
         */
        confirm(options) {
            return BgLoc.confirmReception(options);
        }
        /** Provides an event based update mechanism. */
        on(event) {
            switch (event) {
                case exports.InfaBgLocEvent.LOCATION:
                    return this.location;
                case exports.InfaBgLocEvent.TIMEOUT:
                    return this.idleTimeout;
            }
        }
        /** Requests the required permissions, just in case, that they are not granted already. */
        async requestPermissions() {
            await BgLoc.requestPermissions();
        }
        /** Answers true, when all required permissions are already granted. */
        async checkPermissions() {
            const status = await BgLoc.checkPermissions();
            return status.location === 'granted';
        }
    }

    exports.BgLocWeb = BgLocWeb;
    exports.InfaBgLocService = InfaBgLocService;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({}, capacitorExports, rxjs);
//# sourceMappingURL=plugin.js.map
